#!/usr/bin/python

#
# Volume+ as Camera Button, Deluxe Edition
# 2012-02-19; Thomas Perl <thp.io/about>
# All rights reserved.
#


from QmSystem import *
from PySide.QtCore import *
from PySide.QtGui import *
from PySide.QtDeclarative import *

import sys
import subprocess
import re
import time

app = QApplication(sys.argv)
settings = QSettings('thp.io', 'camerra')
keys = QmKeys()
last_time = time.time()

WAIT_BETWEEN_CLICKS = 3
WINID_REGEX = r'_NET_ACTIVE_WINDOW\(WINDOW\): window id # (0x[0-9a-f]*)'
CAMERA_REGEX = r'WM_CLASS\(STRING\) = "camera-ui"'

def camera_app_on_top():
    # Is the window of the camera app currently on top or some other app?
    root = subprocess.Popen(['xprop', '-root'], stdout=subprocess.PIPE)
    stdout, stderr = root.communicate()
    win_id = re.search(WINID_REGEX, stdout).group(1)

    win_info = subprocess.Popen(['xprop', '-id', win_id],
            stdout=subprocess.PIPE)
    stdout, stderr = win_info.communicate()

    return re.search(CAMERA_REGEX, stdout) is not None

def boolsetting(key, default=False):
    global settings
    # Kludge -> at startup we get 'false' instead of False
    return (settings.value(key, default) not in ('false', False))

def keyEvent(key, state):
    # Callback for QmKeys key events
    global last_time

    if time.time() - last_time < WAIT_BETWEEN_CLICKS:
        # Avoid hammering the cam app when Vol+ is being kept pressed
        return

    if key == state == 2 and camera_app_on_top():
        last_time = time.time()
        do_focus = ['xresponse', '-d', '820x240,1500,820x240', '-w', '2']
        no_focus = ['xresponse', '-d', '820x240,820x240', '-w', '1']
        user_wants_focus = boolsetting('withFocus', False)
        p = subprocess.Popen(do_focus if user_wants_focus else no_focus,
                             stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        p.wait()

class SettingsProxy(QObject):
    def __init__(self, settings):
        QObject.__init__(self)
        self._settings = settings

    @Slot(str, result='QVariant')
    def get(self, key):
        return self._settings.value(key)

    @Slot(str, 'QVariant')
    def set(self, key, value):
        self._settings.setValue(key, value)

class Camerra(QObject):
    def __init__(self):
        QObject.__init__(self)

    @Slot()
    def launchCameraUI(self):
        subprocess.Popen(['camera-ui', '&'], shell=True)

    def oninit(self):
        if boolsetting('startupLaunch', False):
            self.launchCameraUI()

keys.keyEvent.connect(keyEvent)

view = QDeclarativeView()
rootContext = view.rootContext()

proxy = SettingsProxy(settings)
rootContext.setContextProperty('settings', proxy)

camerra = Camerra()
rootContext.setContextProperty('camerra', camerra)

view.setSource('/opt/camerra/camerra.qml')
view.showFullScreen()

camerra.oninit()

app.exec_()

